@@ -11,6 +11,7 @@ import requests
 import six
 import time
 import warnings
+from six.moves.urllib.parse import quote as q
 from io import BytesIO
 from itertools import chain
 from hashlib import sha256
@@ -56,6 +57,8 @@ _OPTIONAL_HEADERS = {
     "checksum": "Mex-Content-Checksum"
 }
 
+_BOOLEAN_HEADERS = {"compressed", "encrypted"}
+
 _RECEIVE_HEADERS = {
     "sender": "Mex-From",
     "recipient": "Mex-To",
@@ -139,7 +142,7 @@ class MeshClient(object):
         self._session = requests.Session()
         self._session.headers = {
             "mex-ClientVersion": "mesh_client=={}".format(VERSION),
-            "mex-OSArchitecture": platform.processor(),
+            "mex-OSArchitecture": platform.processor() or platform.machine(),
             "mex-OSName": platform.system(),
             "mex-OSVersion": "{} {}".format(platform.release(), platform.version()),
             "mex-JavaVersion": "N/A",
@@ -174,7 +177,7 @@ class MeshClient(object):
         List all messages in user's inbox. Returns a list of message_ids
         """
         response = self._session.post(
-            "{}/messageexchange/{}".format(self._url, self._mailbox),
+            "{}/messageexchange/{}".format(self._url, q(self._mailbox)),
             timeout=self._timeout
         )
 
@@ -187,7 +190,7 @@ class MeshClient(object):
         Count all messages in user's inbox. Returns an integer
         """
         response = self._session.get(
-            "{}/messageexchange/{}/count".format(self._url, self._mailbox),
+            "{}/messageexchange/{}/count".format(self._url, q(self._mailbox)),
             timeout=self._timeout)
         response.raise_for_status()
         return response.json()["count"]
@@ -201,9 +204,9 @@ class MeshClient(object):
             raise ValueError("Exactly one of local message id (called tracking_id, for historical reasons) and message_id must be provided")
 
         if tracking_id:
-            url = "{}/messageexchange/{}/outbox/tracking/{}".format(self._url, self._mailbox, tracking_id)
+            url = "{}/messageexchange/{}/outbox/tracking/{}".format(self._url, q(self._mailbox), q(tracking_id))
         else:
-            url = "{}/messageexchange/{}/outbox/tracking?messageId={}".format(self._url, self._mailbox, message_id)
+            url = "{}/messageexchange/{}/outbox/tracking?messageID={}".format(self._url, q(self._mailbox), q(message_id))
 
         response = self._session.get(url, timeout=self._timeout)
         response.raise_for_status()
@@ -215,7 +218,7 @@ class MeshClient(object):
         Returns a dictionary, in much the same format that MESH provides it.
         """
         response = self._session.get(
-            "{}/endpointlookup/mesh/{}/{}".format(self._url, organisation_code, workflow_id),
+            "{}/endpointlookup/mesh/{}/{}".format(self._url, q(organisation_code), q(workflow_id)),
             timeout=self._timeout)
         response.raise_for_status()
         return response.json()
@@ -225,7 +228,7 @@ class MeshClient(object):
         List all messages in user's inbox. Returns a list of message_ids
         """
         response = self._session.get(
-            "{}/messageexchange/{}/inbox".format(self._url, self._mailbox),
+            "{}/messageexchange/{}/inbox".format(self._url, q(self._mailbox)),
             timeout=self._timeout)
         response.raise_for_status()
         return response.json()["messages"]
@@ -237,7 +240,7 @@ class MeshClient(object):
         """
         message_id = getattr(message_id, "_msg_id", message_id)
         response = self._session.get(
-            "{}/messageexchange/{}/inbox/{}".format(self._url, self._mailbox, message_id),
+            "{}/messageexchange/{}/inbox/{}".format(self._url, q(self._mailbox), q(message_id)),
             stream=True,
             timeout=self._timeout)
         response.raise_for_status()
@@ -245,7 +248,7 @@ class MeshClient(object):
 
     def retrieve_message_chunk(self, message_id, chunk_num):
         response = self._session.get(
-            "{}/messageexchange/{}/inbox/{}/{}".format(self._url, self._mailbox, message_id, chunk_num),
+            "{}/messageexchange/{}/inbox/{}/{}".format(self._url, q(self._mailbox), q(message_id), chunk_num),
             stream=True,
             timeout=self._timeout)
         response.raise_for_status()
@@ -293,11 +296,14 @@ class MeshClient(object):
             "Mex-From": self._mailbox,
             "Mex-To": recipient,
             "Mex-MessageType": 'DATA',
-            "Mex-Version": '1.0'
+            "Mex-Version": '1.0',
+            "Content-Type": "application/octet-stream",
         }
 
         for key, value in kwargs.items():
             if key in _OPTIONAL_HEADERS:
+                if key in _BOOLEAN_HEADERS:
+                    value = 'Y' if value else 'N'
                 headers[_OPTIONAL_HEADERS[key]] = str(value)
             else:
                 raise TypeError("Unrecognised keyword argument {key}."
@@ -308,7 +314,7 @@ class MeshClient(object):
                                     ] + list(_OPTIONAL_HEADERS.keys()))))
 
         if transparent_compress:
-            headers["Mex-Content-Compress"] = "TRUE"
+            headers["Mex-Content-Compress"] = "Y"
             headers["Content-Encoding"] = "gzip"
 
         chunks = SplitStream(data, self._max_chunk_size)
@@ -317,10 +323,13 @@ class MeshClient(object):
 
         chunk1 = maybe_compressed(six.next(chunk_iterator))
         response1 = self._session.post(
-            "{}/messageexchange/{}/outbox".format(self._url, self._mailbox),
+            "{}/messageexchange/{}/outbox".format(self._url, q(self._mailbox)),
             data=chunk1,
             headers=headers,
             timeout=self._timeout)
+        # MESH server dumps XML SOAP output on internal server error
+        if response1.status_code >= 500:
+            response1.raise_for_status()
         json_resp = response1.json()
         if response1.status_code == 417 or "errorDescription" in json_resp:
             raise MeshError(json_resp["errorDescription"], json_resp)
@@ -343,7 +352,7 @@ class MeshClient(object):
                 "Mex-From": self._mailbox,
             }
             if transparent_compress:
-                headers["Mex-Content-Compress"] = "TRUE"
+                headers["Mex-Content-Compress"] = "Y"
                 headers["Content-Encoding"] = "gzip"
 
             response = None
@@ -356,7 +365,7 @@ class MeshClient(object):
 
                 response = self._session.post(
                     "{}/messageexchange/{}/outbox/{}/{}".format(
-                        self._url, self._mailbox, message_id, chunk_num),
+                        self._url, q(self._mailbox), q(message_id), chunk_num),
                     data=buf,
                     headers=headers,
                     timeout=self._timeout)
@@ -376,7 +385,7 @@ class MeshClient(object):
         message_id = getattr(message_id, "_msg_id", message_id)
         response = self._session.put(
             "{}/messageexchange/{}/inbox/{}/status/acknowledged".format(
-                self._url, self._mailbox, message_id),
+                self._url, q(self._mailbox), q(message_id)),
             timeout=self._timeout)
         response.raise_for_status()
 
@@ -459,9 +468,9 @@ class Message(object):
 
         for key, value in _RECEIVE_HEADERS.items():
             header_value = headers.get(value, None)
-            if key in ["compressed", "encrypted"]:
-                header_value = header_value or "FALSE"
-                header_value = header_value.upper() == "TRUE"
+            if key in _BOOLEAN_HEADERS:
+                header_value = header_value or "N"
+                header_value = header_value.upper() in ["Y", "TRUE"]
             setattr(self, key, header_value)
         chunk, chunk_count = map(
             int, headers.get("Mex-Chunk-Range", "1:1").split(":"))
@@ -568,7 +577,7 @@ class AuthTokenGenerator(object):
             return token
 
     def generate_token(self):
-        now = datetime.datetime.now().strftime("%Y%m%d%H%M")
+        now = datetime.datetime.utcnow().strftime("%Y%m%d%H%M")
         public_auth_data = _combine(self._mailbox, self._nonce,
                                     self._nonce_count, now)
         private_auth_data = _combine(self._mailbox, self._nonce,
